const rl = @import("raylib.zig");

pub const Input = struct {
    name: []const u8,
    code: i32,
    default: i32,

    pub fn isDown(self: Input) bool {
        return rl.isKeyDown(self.code);
    }
    pub fn isJustPressed(self: Input) bool {
        return rl.isKeyPressed(self.code);
    }
    pub fn isJustReleased(self: Input) bool {
        return rl.isKeyReleased(self.code);
    }
};

fn init(comptime name: []const u8, comptime key_code: []const u8) Input {
    return Input{
        .name = name,
        .code = rl.getKeyCode(key_code),
        .default = rl.getKeyCode(key_code),
    };
}

pub const menu_right = init("Menu Right", "RIGHT");
pub const menu_left = init("Menu Left", "LEFT");
pub const menu_down = init("Menu Down", "DOWN");
pub const menu_up = init("Menu Up", "UP");
pub const menu_accept = init("Menu Accept", "ENTER");
pub const menu_cancel = init("Menu Cancel", "BACKSPACE");

pub const game_right = init("Move Right", "D");
pub const game_left = init("Move Left", "A");
pub const game_down = init("Move Down", "S");
pub const game_drop = init("Drop Piece", "W");
pub const game_swap_piece = init("Swap Piece", "SPACE");
pub const game_rotate_r = init("Rotate Piece Clockwise", "RIGHT");
pub const game_rotate_l = init("Rotate Piece Counterclockwise", "LEFT");
