pub const c = @cImport({
    @cInclude("raylib.h");
});

const std = @import("std");

//const Self = @This();

pub fn initWindow(width: i32, height: i32, title: [*c]const u8, fps: i32) void {
    c.InitWindow(width, height, title);
    c.SetTargetFPS(fps);
}

pub fn windowShouldClose() bool {
    return c.WindowShouldClose();
}

pub fn closeWindow() void {
    c.CloseWindow();
}

pub fn beginDrawing() void {
    c.BeginDrawing();
}

pub fn endDrawing() void {
    c.EndDrawing();
}

pub fn getKeyCode(comptime name: []const u8) i32 {
    const full_name = "KEY_" ++ name;

    if (!@hasDecl(c, full_name)) {
        @compileError(full_name ++ " keycode does not exist.");
    }

    return @field(c, full_name);
}

pub fn isKeyPressed(key: i32) bool {
    return c.IsKeyPressed(key);
}

pub fn isKeyDown(key: i32) bool {
    return c.IsKeyDown(key);
}

pub fn isKeyReleased(key: i32) bool {
    return c.IsKeyReleased(key);
}

pub fn drawText(text: [*c]const u8, x: i32, y: i32, fontSize: i32, color: [4]u8) void {
    c.DrawText(text, x, y, fontSize, .{ .r = color[0], .g = color[1], .b = color[2], .a = color[3] });
}

pub fn clearBackground(color: [4]u8) void {
    c.ClearBackground(.{ .r = color[0], .g = color[1], .b = color[2], .a = color[3] });
}

pub fn drawTriangle(v1: [2]f32, v2: [2]f32, v3: [2]f32, color: [4]u8) void {
    c.DrawTriangle(
        .{ .x = v1[0], .y = v1[1] },
        .{ .x = v2[0], .y = v2[1] },
        .{ .x = v3[0], .y = v3[1] },
        .{ .r = color[0], .g = color[1], .b = color[2], .a = color[3] },
    );
}

pub const Texture = struct {
    texture: c.Texture2D,

    pub fn load(file_name: [*c]const u8) Texture {
        const aux = .{
            .texture = c.LoadTexture(file_name),
        };
        c.SetTextureFilter(aux.texture, c.TEXTURE_FILTER_BILINEAR);
        return aux;
    }

    pub fn drawTo(self: Texture, x: i32, y: i32, w: i32, h: i32, tint: [4]u8) void {
        std.debug.print("\n\nachooooo\n\n", .{});
        const srcRec: c.Rectangle = .{
            .x = 0,
            .y = 0,
            .width = @floatFromInt(self.texture.width),
            .height = @floatFromInt(self.texture.height),
        };
        std.debug.print("\n\nnaguru\n\n", .{});

        const dstRec: c.Rectangle = .{
            .x = @floatFromInt(x),
            .y = @floatFromInt(y),
            .width = @floatFromInt(w),
            .height = @floatFromInt(h),
        };

        std.debug.print("\n\nHAstta aqui hemos llegao\n\n", .{});
        c.DrawTexturePro(
            self.texture,
            srcRec,
            dstRec,
            .{ .x = 0, .y = 0 },
            0,
            .{ .r = tint[0], .g = tint[1], .b = tint[2], .a = tint[3] },
        );
        std.debug.print("\n\nlmaolol\n\n", .{});
    }

    pub fn unload(self: Texture) void {
        c.UnloadTexture(self.texture);
    }
};

pub fn getScreenWidth() i32 {
    return c.GetScreenWidth();
}

pub fn getScreenHeight() i32 {
    return c.GetScreenHeight();
}

pub fn getTime() f64 {
    return c.GetTime();
}

pub fn drawFPS(x: i32, y: i32) void {
    c.DrawFPS(x, y);
}
